#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

from sys import path as PATH
from os.path import exists
from os.path import expanduser
from os.path import sep as SEP
from os import mkdir
import shelve
import gobject

SRC_PATH = SEP.join(__file__.split(SEP)[:-2])
PATH.append(SRC_PATH)

from hosts import HostList

HOME = expanduser('~') + SEP + '.beholder'
if not exists(HOME):
    mkdir(HOME)

HOSTS_DB = HOME + SEP + 'hosts.db'
GROUPS_DB = HOME + SEP + 'groups.db'
CONFIGS_DB = HOME + SEP + 'configs.db'


class Database(gobject.GObject):
    __gsignals__ = {
                    # Hosts Signals
                    'new-host'          : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'removed-host'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'updated-host'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),

                    #Groups signals
                    'new-group'         : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'removed-group'     : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'updated-group'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                   }
    def __init__ (self):
        gobject.GObject.__init__(self)
        self.hosts = None
        self.groups = None

        print ' * Opening hosts DB...'
        self.hosts_db = shelve.open(HOSTS_DB)
        #self.groups_db = shelve.open(GROUPS_DB)
        self.open_hosts()

    def open_hosts(self):
        print ' * Loading hosts...'
        hosts_dic = dict(self.hosts_db)
        print ' * Processing hosts...'
        self.hosts = HostList(hosts_dic)
        self.hosts.connect('host-added', self.__on_host_sig, 'new-host')
        self.hosts.connect('host-removed', self.__on_host_sig, 'removed-host')
        self.hosts.connect('host-updated', self.__on_host_sig, 'updated-host')
        print ' * Done!'

        print ' * The following hosts was loaded:'
        for host in self.hosts:
            print '\t',host.hostname

    def update_hosts(self):
        self.hosts_db.update(self.hosts.as_dict())
        self.sync()

    def sync(self):
        print ' * Syncing hosts DB'
        self.hosts_db.sync()

    def close(self):
        print ' * Closing hosts DB'
        self.update_hosts()
        self.hosts_db.close()

    def __del__(self):
        self.sync()
        self.close()

    def __on_host_sig(self, hosts, host, msg):
        if msg == 'removed-host':
            print ' * Removing host:',host.hostname
            self.hosts_db.pop(host.hostname)
        self.update_hosts()
        self.emit(msg, host)
