#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject

import beholderlib as lib

_ = lib._
hosts = lib.db.hosts

class HostnameEntry(gtk.Entry):
    __gsignals__ = {
                    'changed-hostname'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'alert-hostname'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_BOOLEAN,)),
                   }
    def __init__(self):
        gtk.Entry.__init__(self)
        self.connect("key-release-event", self.__on_hostname_change)
        self.is_valid = False

    def __on_hostname_change(self, widget, event):
        hostname = widget.get_text()
        if widget.get_text() == "":
            self.is_valid = False
        elif widget.get_text() in hosts:
            self.is_valid = False
            self.emit('alert-hostname', True)
        else:
            self.is_valid = True
            self.emit('alert-hostname', False)
        self.emit('changed-hostname')


class HostAddress(gtk.Entry):
    __gsignals__ = {
                    'changed-address'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
                   }
    def __init__(self):
        gtk.Entry.__init__(self)
        self.connect("key-release-event", self.__on_address_change)
        self.is_valid = False

    def __on_address_change(self, widget, event):
        if widget.get_text() == "":
            self.is_valid = False
        else:
            self.is_valid = True
        self.emit('changed-address')


class HostTags(gtk.Frame):
    def __init__(self):
        gtk.Frame.__init__(self)
        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)

        self.tags_tb = gtk.TextBuffer()

        self.tags_tv = gtk.TextView(self.tags_tb)
        self.tags_tv.set_wrap_mode(gtk.WRAP_WORD)
        self.add(self.tags_tv)

    def load_tags(self, tags):
        if tags == []:
            return
        self.tags_tb.set_text(', '.join(tags))

    def get_tags(self):
        start_iter = self.tags_tb.get_start_iter()
        end_iter = self.tags_tb.get_end_iter()
        tags_text = self.tags_tb.get_text(start_iter, end_iter, include_hidden_chars=False)
        tags = [tag.strip() for tag in tags_text.split(',') if tag.strip() != '']
        tags = list(set(tags))
        return tags

