#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject

import beholderlib as lib

_ = lib._
SEP = lib.SEP
ICONS_PATH = lib.ICONS_PATH

CONN_PRETTY_NAME = lib.hosts.CONN_PRETTY_NAME
SUPPORTED_CONN = lib.hosts.SUPPORTED_CONN


class ProtocolComboBox(gtk.ComboBox):
    __gsignals__ = {
                'protocol-changed'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))
               }
    def __init__(self, ports=True):
        # TODO: Add icons
        self.ports = ports
        if self.ports:
            self.contents = gtk.ListStore(str, str)
        else:
            self.contents = gtk.ListStore(str)
        gtk.ComboBox.__init__(self, self.contents)

        cell = gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, 'text', 0)

        if self.ports:
            cell = gtk.CellRendererText()
            self.pack_start(cell, False)
            self.add_attribute(cell, 'text', 1)

        service_list = SUPPORTED_CONN.items()
        service_list.sort()
        for n,(service, port) in enumerate(service_list):
            if ports:
                self.contents.append((CONN_PRETTY_NAME[service], "("+str(port)+"/TCP)"))
            else:
                self.contents.append([CONN_PRETTY_NAME[service]])
            if service == "telnet":
                self.set_active(n)

        self.connect('changed', self.__on_protocol_change)

    def get_selected(self):
        sel_pretty_name = self.get_active_text()
        sel_service = None
        sel_port = None
        for service, pretty_name in CONN_PRETTY_NAME.items():
            if sel_pretty_name == pretty_name:
                sel_service = service
                sel_port = SUPPORTED_CONN[service]
        if self.ports:
            return (sel_service, sel_port)
        else:
            return sel_service

    def set_protocol(self, type):
        model = self.get_model()
        for content in self.contents:
            if CONN_PRETTY_NAME[type] == model.get(content.iter, 0)[0]:
                self.set_active_iter(content.iter)

    def __on_protocol_change(self, widget):
        if self.ports:
            self.emit('protocol-changed', self.get_selected()[0])
        else:
            self.emit('protocol-changed', self.get_selected())


class PortSpin(gtk.SpinButton):
    def __init__(self, host=None):
        self.host = host
        adj = gtk.Adjustment(value=1, lower=1, upper=65535, step_incr=1, page_incr=10, page_size=0)
        gtk.SpinButton.__init__(self, adjustment=adj, climb_rate=1.0, digits=0)

    def set_port(self, port):
        self.set_value(float(port))

    def get_port(self):
        return str(self.get_value_as_int())

    def update_with_protocol(self, widget, service):
        if service == 'www':
            self.set_sensitive(False)
        else:
            self.set_sensitive(True)
        self.set_port(SUPPORTED_CONN[service])
