#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#


import gtk
import pygtk
import gobject
from gtkmozembed import MozEmbed as MozEmbedView
from gtkmozembed import FLAG_RELOADNORMAL
from webkit import WebView as WebKitView

gobject.threads_init()

class WebKit(WebKitView):
    def __init__(self, toolbar, label):
        WebKitView.__init__(self)
        self.label = label
        self.toolbar = toolbar
        
        self.toolbar.stop_state(False)
        self.toolbar.forward_state(False)
        self.toolbar.back_state(False)
        
        self.connect('title-changed', self.__on_title_update)
        self.connect('load-progress-changed', self.__on_load_change)
        self.connect('load-started', self.__on_load_start)
        self.connect('load-finished', self.__on_load_finish)
        
        nav_stop = lambda widget : self.stop_loading()
        nav_back = lambda widget : self.go_back()
        nav_forward = lambda widget : self.go_forward()
        nav_reload = lambda widget : self.reload_page()
        nav_goto = lambda widget,url : self.open(url)
#        
        self.toolbar.connect('stop', nav_stop)
        self.toolbar.connect('back', nav_back)
        self.toolbar.connect('forward', nav_forward)
        self.toolbar.connect('reload', nav_reload)
        self.toolbar.connect('go-to',nav_goto)

    def reload_page(self):
        # Created becouse the webkit.WebView.reload() and 
        # webkit.WebView.reload_bypass_cache() sometimes doesn't work.
        self.open(self.toolbar.uri_entry.get_text())
        
    def __on_load_change(self, widget, progress):
        self.toolbar.set_progress_fraction(float(progress)/100)
        
    def __on_load_start(self, widget, frame):
        self.toolbar.stop_state(True)
        self.toolbar.working(True)
        self.toolbar.back_state(self.can_go_back())
        self.toolbar.forward_state(self.can_go_forward())
        
    def __on_load_finish(self, widget, frame):
        self.toolbar.set_progress_fraction(0)
        self.toolbar.working(False)
        self.toolbar.stop_state(False)
        self.toolbar.back_state(self.can_go_back())
        self.toolbar.forward_state(self.can_go_forward())
        
    def __on_title_update(self, widget, frame, title):
        self.label.set_label(title)
        self.toolbar.set_uri(self.get_property('uri'))
        
    def connect_to(self, uri):
        if 'http' not in uri:
            self.open('http://'+uri)
            self.toolbar.set_uri('http://'+uri)
        else:
            self.open(uri)
            self.toolbar.set_uri(uri)
        self.toolbar.working(True)
        
class MozEmbed(MozEmbedView):
    def __init__(self, toolbar, label):
        MozEmbedView.__init__(self)
        self.label = label
        self.toolbar = toolbar
        
        self.toolbar.stop_state(False)
        self.toolbar.forward_state(False)
        self.toolbar.back_state(False)
        
        self.connect('title', self.__on_title_update)
        self.connect('open-uri', self.__on_open_uri)
        self.connect('progress', self.__on_progress_update)
        self.connect('net-stop', self.__on_stop)
        
        nav_stop = lambda widget : self.stop_load()
        nav_back = lambda widget : self.go_back()
        nav_forward = lambda widget : self.go_forward()
        nav_reload = lambda widget : self.reload(gtkmozembed.FLAG_RELOADNORMAL)
        nav_goto = lambda widget,url : self.load_url(url)
        
        self.toolbar.connect('stop', nav_stop)
        self.toolbar.connect('back', nav_back)
        self.toolbar.connect('forward', nav_forward)
        self.toolbar.connect('reload', nav_reload)
        self.toolbar.connect('go-to',nav_goto)
        
    def __on_title_update(self, widget):
        self.label.set_label(self.get_title())
        
    def __on_open_uri(self, widget, uri):
        if uri == "about:blank":
            return
        self.toolbar.set_uri(uri)
        
    def __on_progress_update(self, widget, curr, max):
        self.toolbar.stop_state(True)
        self.toolbar.back_state(self.can_go_back())
        self.toolbar.forward_state(self.can_go_forward())
        
        fraction = float(curr)/max
        self.toolbar.set_progress_fraction(fraction)
        
    def __on_stop(self, widget):
        self.toolbar.set_progress_fraction(0)
        self.toolbar.stop_state(False)
        self.toolbar.back_state(self.can_go_back())
        self.toolbar.forward_state(self.can_go_forward())
        
    def connect_to(self, uri):
        self.load_url(uri)
