#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject

import beholderlib as lib

_ = lib._
ICONS_PATH = lib.ICONS_PATH
SEP = lib.SEP
ICONS_PATH = ICONS_PATH + SEP + "24x24" + SEP


class HostPopMenu(gtk.Menu):
    __gsignals__ = {
                    'host-properties'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'remove-host'        : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'add-host'        : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-add-host'        : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    # Quicly Edit Submenu Signals
                    'quickly-edit-addres'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-tags'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-username'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-protocol'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-icon'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                   }
    def __init__(self):
        gtk.Menu.__init__(self)
        
        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
        self.add(add, 'add-host')
        
        quickly_add = gtk.MenuItem(_('Quickly Add Host'))
        self.add(quickly_add, 'quickly-add-host')
        
        prop = gtk.ImageMenuItem(gtk.STOCK_PROPERTIES)
        self.add(prop, 'host-properties')
        
#        submenu_sigs = [
#                        'quickly-edit-addres',
#                        'quickly-edit-tags',
#                        'quickly-edit-username',
#                        'quickly-edit-protocol',
#                        'quickly-edit-icon',
#                       ]
#        self.add_sub_menu(_('Quickly Edit'), QuicklyEditSubMenu(), submenu_sigs)
        
        remove = gtk.ImageMenuItem(gtk.STOCK_REMOVE)
        self.add(remove, 'remove-host')

        self.show_all()
        
    def my_emit(self, widget, event, signal):
        self.emit(signal)
    
    def add(self, menu_item, sig_name):
        gtk.Menu.add(self, menu_item)
        menu_item.connect('button-press-event', self.my_emit, sig_name)
        
    def add_sub_menu(self, submenu_name, submenu, signals):
        submenu_holder = gtk.MenuItem(submenu_name)
        submenu_holder.set_submenu(submenu)
        gtk.Menu.add(self, submenu_holder)
        
        for sig_name in signals:
            submenu.connect(sig_name, self.my_emit, None, sig_name)
        
        
class QuicklyEditSubMenu(gtk.Menu):
    __gsignals__ = {
                    'quickly-edit-addres'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-tags'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-username'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-protocol'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-edit-icon'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                   }
    def __init__(self):
        gtk.Menu.__init__(self)
        
        address = gtk.MenuItem(_('Address'))
        self.add(address, 'quickly-edit-addres')
        
        tags = gtk.MenuItem(_('Tags'))
        self.add(tags, 'quickly-edit-tags')
        
        username = gtk.MenuItem(_('Username'))
        self.add(username, 'quickly-edit-username')
        
        protocol = gtk.MenuItem(_('Protocol & Port'))
        self.add(protocol, 'quickly-edit-protocol')
        
        icon = gtk.MenuItem(_('Icon'))
        self.add(icon, 'quickly-edit-icon')
        
        self.show_all()
        
    def my_emit(self, widget, event, signal):
        self.emit(signal)
        
    def add(self, menu_item, sig_name):
        gtk.Menu.add(self, menu_item)
        menu_item.connect('button-press-event', self.my_emit, sig_name)


host_popupmenu = HostPopMenu()
