#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject
import gtkmozembed

import vte
from pango import FontDescription
from pango import ELLIPSIZE_MIDDLE

import toolbars
import browsers

from os import sep as SEP
from beholderlib import ICONS_PATH
CLOSE_ICON = ICONS_PATH + SEP + '16x16' + SEP + 'tag-close.png'
STARTUP_IMG = ICONS_PATH + SEP + 'dashboard.png'

class ConnNotebook(gtk.Notebook):
    def __init__(self):
        gtk.Notebook.__init__(self)
        self.set_show_tabs(False)
        self.set_scrollable(True)

        #self.tabs = []
        self.startup = StartupTab()
        self.append_page(self.startup)
        self.show_all()

    def change_page(self, widget, index):
        self.set_current_page(index)

    def open_connection(self, widget, host):
        label = TabLabel(host)

        if host.connection.type == 'www':
            conn_tab = WebTab(host, label)
            self.append_page(conn_tab, label)
        else:
            conn_tab = ConnectionTab(host, label)
            scroll = gtk.ScrolledWindow()
            scroll.add_with_viewport(conn_tab)
            scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            scroll.show_all()
            self.append_page(scroll, label)

        conn_tab.connect('tab-closed', self.__on_tab_closed)
        
        self.set_current_page(-1)

        if self.get_n_pages() > 1:
            self.startup.hide()
            self.set_show_tabs(True)
        else:
            self.set_show_tabs(False)

        conn_tab.connect_to_host()
        conn_tab.grab_focus()

    def __on_tab_closed(self, tab):
        n = self.page_num(tab)
        if self.get_n_pages() == 2:
            self.set_show_tabs(False)
            self.startup.show_all()
        self.remove_page(n)


class TabLabel(gtk.HBox):
    def __init__(self, host):
        gtk.HBox.__init__(self)
        self.button = gtk.Button()

        image = gtk.image_new_from_file(CLOSE_ICON)
        self.button.add(image)
        self.button.set_relief(gtk.RELIEF_NONE)

        width, height = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
        # TODO: Fix this
        self.button.set_size_request(width + 8, height + 8)
        self.label = gtk.Label(host.connection.type+' | '+host.hostname)
        self.label.props.max_width_chars = 25
        self.label.set_ellipsize(ELLIPSIZE_MIDDLE)
        self.label.set_alignment(0.0, 0.5)
        self.pack_start(self.label)
        self.pack_start(self.button, True, False, 0)
        self.show_all()
        
    def set_label(self, label_text):
        self.label.set_text(label_text)


class ConnectionTab(vte.Terminal):
    __gsignals__ = {
                    'tab-closed'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                   }
    def __init__(self, host, label):
        vte.Terminal.__init__(self)
        self.host = host
        label.button.connect('clicked', self.__on_close_bnt)

        self.set_font(FontDescription("Monospace 10"))
        self.set_background_transparent(False)
        self.connect('child-exited',self.__on_term_exit)
        self.show()

    def connect_to_host(self):
        command = self.host.get_command()
        command = command.split(' ')
        self.fork_command(command[0], command)

    def __on_close_bnt(self, button):
        self.emit("tab-closed")
        self.destroy()

    def __on_term_exit(self, widget):
        status = self.get_child_exit_status()
        if status == 0:
            self.emit("tab-closed")
            
class WebTab(gtk.VBox):
    __gsignals__ = {
                    'tab-closed'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                   }
    def __init__(self, host, label):
        gtk.VBox.__init__(self)
        
        self.host = host
        self.label = label
        label.button.connect('clicked', self.__on_close_bnt)
        
        self.toolbar = toolbars.WebToolbar()
        self.pack_start(self.toolbar, expand=False)
        
        self.browser = browsers.WebKit(self.toolbar, self.label)
        #self.browser = browsers.MozEmbed(self.toolbar, self.label)
        scroll = gtk.ScrolledWindow()
        scroll.add_with_viewport(self.browser)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.show_all()
        self.pack_start(scroll)
        
        self.show_all()
        
    def __on_close_bnt(self, widget):
        self.browser.destroy()
        self.emit("tab-closed")
        self.destroy()
        
    def connect_to_host(self):
        self.browser.connect_to(self.host.address)
    

class StartupTab(gtk.VBox):
    def __init__(self):
        gtk.VBox.__init__(self)
        self.set_spacing(10)
        self.set_border_width(10)
        self.pagetype = 'startup'
        self.pageitem = None
        self.pixbuf = gtk.gdk.pixbuf_new_from_file(STARTUP_IMG)
        self.image = gtk.Image()
        self.image.connect('expose-event', self.__on_expose)
        self.pack_start(self.image, expand=False)
        self.last_width = 0

        self.add_main_credits("""<b>Code and Art by:</b>
    Magnun Leno da Silva
<b>Created by:</b>
    Magnun Leno da Silva
    Thomas Acauan Schertel""")

        self.release_vbox = gtk.VBox()
        self.scroll = gtk.ScrolledWindow()
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.pack_start(self.scroll)
        self.scroll.add_with_viewport(self.release_vbox)
        self.add_release("<b>Release 0.1</b>", """* Host management dashboard;
* Telnet connections;
* SSh connections;
* Alphabetically sorted hosts view;
* Hosts tags view;""")

    def add_main_credits(self, description):
        desc_label = gtk.Label()
        desc_label.set_markup(description)
        desc_label.set_property('xalign', 0.0)
        self.pack_start(desc_label, expand=False)

    def add_release(self, release_name, description):
        name_label = gtk.Label()
        desc_label = gtk.Label()
        name_label.set_markup(release_name)
        desc_label.set_markup(description)
        name_label.set_property('xalign', 0)
        desc_label.set_property('xalign', 0.05)
        self.release_vbox.pack_start(name_label, expand=False)
        self.release_vbox.pack_start(desc_label, expand=False)

    def __on_expose(self, widget, event):
        ratio = float(event.area.width)/float(self.pixbuf.get_width())
        w = int(event.area.width)
        if w != self.last_width:
            self.last_width = w
        else:
            return
        h = int(self.pixbuf.get_height()*ratio)

        if 50 < w < 5000:
            new_pix = self.pixbuf.scale_simple(w,h,gtk.gdk.INTERP_BILINEAR)
            self.image.set_from_pixbuf(new_pix)
        else:
            self.image.set_from_pixbuf(self.pixbuf)
