#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject

import beholderlib as lib

_ = lib._
ICONS_PATH = lib.ICONS_PATH
SEP = lib.SEP
THROBER_PATH = ICONS_PATH + SEP + "animated" + SEP
ICONS_PATH = ICONS_PATH + SEP + "24x24" + SEP


class SmallToolbar(gtk.Toolbar):
    __gsignals__ = {
                    'add-host'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'quickly-add-host'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'host-properties'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'remove-host'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                   }
    def __init__(self):
        gtk.Toolbar.__init__(self)
        self.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        self.set_style(gtk.TOOLBAR_ICONS)
        self.sensitive_buttons = []

        # Properties
        self.quickl_add = self.add_btn(ICONS_PATH+"quickly.png",
                                       _("Quickly Add Host"),
                                       _("Quickly add a new host to the list"), 
                                       self.__on_signaling, 
                                       'quickly-add-host')
        self.add = self.add_btn(gtk.STOCK_ADD, 
                                _("Add Host"), 
                                _("Add a fully detailed new host to the list"), 
                                self.__on_signaling, 
                                'add-host')
        self.remove = self.add_btn(gtk.STOCK_REMOVE, 
                                   _("Remove Host"), 
                                   _("Remove selected host from the list"), 
                                   self.__on_signaling, 
                                   'remove-host')
        self.properties = self.add_btn(gtk.STOCK_PROPERTIES, 
                                       _("Host Properties"), 
                                       _("Show host properties"), 
                                       self.__on_signaling, 
                                       'host-properties')

        self.sensitive_buttons.append(self.properties)
        self.sensitive_buttons.append(self.remove)

    def add_btn(self, image, label, tip, event_func, sig_name):
        toolbutton = None
        if 'gtk-' in image:
            toolbutton = gtk.ToolButton(image)
        else:
            img = gtk.Image()
            img.set_from_file(image)
            toolbutton = gtk.ToolButton(img, label=label)

        toolbutton.connect("clicked", event_func, sig_name)
        toolbutton.set_tooltip_text(tip)
        self.insert(toolbutton, -1)
        return toolbutton

    def __on_signaling(self, widget, signal):
        self.emit(signal)

    def on_item_selected(self, widget, item):
        if item is None:
            for button in self.sensitive_buttons:
                button.set_sensitive(False)
        else:
            for button in self.sensitive_buttons:
                button.set_sensitive(True)



class WebToolbar(gtk.Toolbar):
    __gsignals__ = {
                    'forward'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'back'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'stop'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'reload'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'go-to'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING,)),
                   }
    def __init__(self):
        gtk.Toolbar.__init__(self)
        self.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        self.set_style(gtk.TOOLBAR_ICONS)
        
        # Properties
        self.back = self.add_btn(gtk.STOCK_GO_BACK, _("Back"), _("Go back"), 
                                 self.__on_signaling, 'back')
                                        
        self.forward = self.add_btn(gtk.STOCK_GO_FORWARD, _("Forward"),
                                    _("Go forward"), self.__on_signaling, 
                                    'forward')
        
        self.reload = self.add_btn(gtk.STOCK_REFRESH, _("Reload"),
                                   _("Reload current page"), 
                                   self.__on_signaling, 'reload')
        
        self.stop = self.add_btn(gtk.STOCK_STOP, _("Stop"),
                                 _("Stop loading the current page"), 
                                 self.__on_signaling,'stop')
        self.uri_entry = self.add_uri_entry()
        self.throbber = self.add_throbber()
        self.pixbuf = self.throbber.get_pixbuf()
        self.animation = gtk.gdk.PixbufAnimation(THROBER_PATH+'working.gif')
        goto_signal = lambda x : self.emit('go-to', self.uri_entry.get_text())
        self.uri_entry.connect('activate', goto_signal)
        
        
    def __on_signaling(self, widget, signal):
        self.emit(signal)
        
    def add_btn(self, image, label, tip, event_func, sig_name):
        toolbutton = None
        if 'gtk-' in image:
            toolbutton = gtk.ToolButton(image)
        else:
            img = gtk.Image()
            img.set_from_file(image)
            toolbutton = gtk.ToolButton(img, label=label)

        toolbutton.connect("clicked", event_func, sig_name)
        toolbutton.set_tooltip_text(tip)
        self.insert(toolbutton, -1)
        return toolbutton
        
    def add_uri_entry(self):
        entry_holder = gtk.ToolItem()
        entry_holder.set_expand(True)
        uri_entry = gtk.Entry()
        entry_holder.add(uri_entry)
        self.insert(entry_holder, -1)
        return uri_entry
        
    def add_throbber(self):
        throbber_holder = gtk.ToolItem()
        throbber = gtk.Image()
        throbber.set_padding(5, 0)
        throbber.set_from_file(THROBER_PATH+'stop.png')
        throbber_holder.add(throbber)
        self.insert(throbber_holder, -1)
        return throbber
        
    def working(self, flag):
        if flag:
            self.throbber.set_from_animation(self.animation)
        else:
            self.throbber.set_from_pixbuf(self.pixbuf)
        
    def stop_state(self, state):
        self.stop.set_sensitive(state)
        
    def forward_state(self, state):
        self.forward.set_sensitive(state)
        
    def back_state(self, state):
        self.back.set_sensitive(state)
        
    def set_uri(self, uri):
        self.uri_entry.set_text(uri)
        
    def set_progress_fraction(self, fraction):
        try:
            self.uri_entry.set_progress_fraction(fraction)
        except:
            pass
