#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject
from pango import ELLIPSIZE_END, ELLIPSIZE_MIDDLE

import beholderlib as lib
import popupmenus

_ = lib._
SEP = lib.SEP
ICONS_PATH = lib.ICONS_PATH

ROOT_TREE_LABEL = _("Connections")
NO_TAG_LABEL = _('No tags')

TREE_ICON_SIZE = "16x16"
TREE_ICONS_PATH = SEP.join([ICONS_PATH, TREE_ICON_SIZE,''])

ROOT_ICON = TREE_ICONS_PATH + 'globe.png'
TAG_ICON = TREE_ICONS_PATH + 'tags.png'
NO_TAG_ICON = TREE_ICONS_PATH + 'no_tags.png'

ITEM_ID = lib.hosts.ITEM_ID
ROOT_ID = lib.hosts.ROOT_ID
GROUP_ID = lib.hosts.GROUP_ID
NO_GROUP_ID = lib.hosts.NO_GROUP_ID
TAG_ID = lib.hosts.TAG_ID
NO_TAG_ID = lib.hosts.NO_TAG_ID

class GenericHostList(gtk.TreeView):
    __gsignals__ = {
                    'open-connection'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'item-selected'        : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                   }
    def __init__(self):
        gtk.TreeView.__init__(self)
        self.hosts = lib.db.hosts
        self.hosts.connect('host-updated', self.__on_host_update, 'update')
        self.hosts.connect('host-added', self.__on_host_update, 'add')
        self.hosts.connect('host-removed', self.__on_host_update, 'remove')
        self.last_width = None
        self.set_headers_visible(False)

    def create_list(self):
        self.add_pxbuf_column("Hostname", 0)
        self.add_text_column("Address", 1)

        self.contents = gtk.TreeStore(str, str, int)
        self.set_model(self.contents)

        self.connect("row-activated", self.on_row_double_click)
        self.connect("button-press-event", self.on_row_click)
        self.connect("button-release-event", self.on_row_click_leave)
        self.populate()

    def add_pxbuf_column(self, title, id, group=False):
        cellpb = gtk.CellRendererPixbuf()
        cell = gtk.CellRendererText()
        cell.set_property('ellipsize', ELLIPSIZE_END)

        column = gtk.TreeViewColumn(title)
        column.set_sort_column_id(0)
        column.pack_start(cellpb, False)
        column.pack_start(cell, True)
        column.set_min_width(180)
        column.set_resizable(True)
        column.set_expand(True)

        column.set_cell_data_func(cellpb, self.host_pixbuf)
        column.set_cell_data_func(cell, self.host_name)
        self.append_column(column)

    def add_text_column(self, title, id, visible=True):
        cell_renderer = gtk.CellRendererText()
        cell_renderer.set_property('ellipsize', ELLIPSIZE_END)
        column = gtk.TreeViewColumn(title, cell_renderer, text=id, markup=0)
        column.set_min_width(100)
        column.set_resizable(True)
        column.set_sort_column_id(id)
        column.set_visible(visible)
        column.set_expand(True)
        self.append_column(column)

    def host_pixbuf(self, column, cell, model, iter):
        pb = None
        icon_name = None
        ID = model.get_value(iter, 2)
        name = model.get_value(iter, 0)
        if ID is ROOT_ID:
            icon_name = ROOT_ICON
        elif ID is ITEM_ID:
            host = self.hosts.get_by_hostname(name)
            icon_name = host.get_small_icon()
        elif ID is TAG_ID:
            icon_name = TAG_ICON
        elif ID is NO_TAG_ID:
            icon_name = NO_TAG_ICON

        pb = gtk.gdk.pixbuf_new_from_file(icon_name)
        cell.set_property('pixbuf', pb)
        cell.set_property('xalign', 0.5)

    def host_name(self, column, cell, model, iter):
        cell.set_property('text', model.get_value(iter, 0))

    def populate(self):
        pass

    def get_selected(self):
        selection = self.get_selection()
        if selection is None:
            return None
        model, selection_iter = selection.get_selected()
        if not selection_iter:
            return None
        return model.get_value(selection_iter,0), model.get_value(selection_iter,1), model.get_value(selection_iter,2)

    def find_host_iter(self, parent_iter, host):
        model = self.get_model()
        child_count = model.iter_n_children(parent_iter)
        for n in range(child_count):
            child_iter = model.iter_nth_child(parent_iter, n)
            if model.get(child_iter, 0)[0] == host.hostname:
                return child_iter
        return None

    def find_tag_iter(self, tag_label):
        model = self.get_model()
        for model_row in self.contents:
            if model.get(model_row.iter, 0)[0] == tag_label:
                return model_row.iter
        return None

    def update_tag_count(self, tag_iter, inc):
        model = self.get_model()
        count = int(model.get(tag_iter, 1)[0].split(':')[-1])
        count = count + inc
        self.contents.set_value(tag_iter, 1, _("Total")+": "+str(count))
        return count

    def add_new_tag(self, tag):
        model = self.get_model()
        tags_count = model.iter_n_children(None)
        for n in range(tags_count):
            tag_iter = model.iter_nth_child(None, n)
            if model.get(tag_iter, 0)[0] == NO_TAG_LABEL:
                continue
            if tag < model.get(tag_iter, 0)[0]:
                return self.contents.insert_before(None, tag_iter, [tag, "Total: 0", TAG_ID])
        else:
            return self.contents.insert_after(None, tag_iter, [tag, "Total: 0", TAG_ID])

    def remove_tag(self, tag):
        if tag == NO_TAG_LABEL:
            return

        model = self.get_model()
        tags_count = model.iter_n_children(None)
        for n in range(tags_count):
            tag_iter = model.iter_nth_child(None, n)
            if model.get(tag_iter, 0)[0] == NO_TAG_LABEL:
                continue
            if tag == model.get(tag_iter, 0)[0]:
                self.contents.remove(tag_iter)
                return

    def find_host_position_at_tag(self, tag_iter, host):
        model = self.get_model()
        child_count = model.iter_n_children(tag_iter)
        for n in range(child_count):
            child_iter = model.iter_nth_child(tag_iter, n)
            if host.hostname < model.get(child_iter, 0)[0]:
                return child_iter
        else:
            return None

    def add_host_at_tag(self, tag, host):
        model = self.get_model()
        tag_iter = self.find_tag_iter(tag)

        child_count = model.iter_n_children(tag_iter)
        if child_count == 0:
            # Tag is empty
            self.contents.append(tag_iter, [host.hostname, host.address, ITEM_ID])
            self.update_tag_count(tag_iter, +1)
            return

        child_iter = self.find_host_position_at_tag(tag_iter, host)
        if child_iter is not None:
            # Isn't the last item
            self.contents.insert_before(tag_iter, child_iter, [host.hostname, host.address, ITEM_ID])
        else:
            # Is the last item
            child_iter = model.iter_nth_child(tag_iter, child_count-1)
            self.contents.insert_after(tag_iter, child_iter, [host.hostname, host.address, ITEM_ID])
        self.update_tag_count(tag_iter, +1)

    def on_row_double_click(self, widget, column = None, pointer = None):
        selected = self.get_selected()
        if selected[-1] == ITEM_ID:
            host = self.hosts.get_by_hostname(selected[0])
            self.emit('open-connection',host)

    def on_row_click(self, widget, event):
        path = widget.get_path_at_pos(int(event.x), int(event.y))
        selection = widget.get_selection()
        if path is None:
            selection.unselect_all()
            self.emit('item-selected', None)

    def on_row_click_leave(self, widget, event):
        selected = self.get_selected()
        if selected is None:
            return

        if selected[-1] == ITEM_ID:
            host = self.hosts.get_by_hostname(selected[0])
            self.emit('item-selected', host)
        else:
            self.emit('item-selected', None)

        if event.button == 3 and selected[-1] == ITEM_ID:
            # Right click
            popupmenus.host_popupmenu.host_data = selected
            popupmenus.host_popupmenu.popup(None, None, None, event.button, event.time)

    def __on_host_update(self, hosts, host, type):
        if type == 'add':
            self.add_single_host(host)
        elif type == 'remove':
            self.remove_single_host(host)
        elif type == 'update':
            self.update_single_host(host)
        else:
            self.populate()

    def add_single_host(self, host):
        pass

    def remove_single_host(self, host):
        pass

    def update_single_host(self, host):
        pass

class SimpleHostsList(GenericHostList):
    def __init__(self):
        GenericHostList.__init__(self)

    def populate(self):
        self.contents.clear()
        self.root_iter = self.contents.append(None, [ROOT_TREE_LABEL, _("Total")+": "+str(len(self.hosts)), ROOT_ID])

        for host in self.hosts:
            self.contents.append(self.root_iter, [host.hostname, host.address, ITEM_ID])

        self.contents.set_sort_column_id(0, gtk.SORT_ASCENDING)
        self.expand_row(path=0, open_all=False)

    def update_root_count(self):
        self.contents.set_value(self.root_iter, 1, _("Total")+": "+str(len(self.hosts)))
        

    def add_single_host(self, host):
        self.contents.append(self.root_iter, [host.hostname, host.address, ITEM_ID])
        self.update_root_count()

    def remove_single_host(self, host):
        host_iter = self.find_host_iter(self.root_iter, host)
        self.contents.remove(host_iter)
        self.update_root_count()

    def update_single_host(self, host):
        host_iter = self.find_host_iter(self.root_iter, host)
        self.contents.set_value(host_iter, 1, host.address)



class GroupHostsList(GenericHostList):
    def __init__(self):
        GenericHostList.__init__(self)

    def populate(self):
        pass

class TagHostsList(GenericHostList):
    def __init__(self):
        GenericHostList.__init__(self)

    def populate(self):
        self.contents.clear()
        tags_dic = {}

        hostnames = [host.hostname for host in self.hosts]
        hostnames.sort()

        for hostname in hostnames:
            host = self.hosts.get_by_hostname(hostname)
            if len(host.tags) == 0:
                if NO_TAG_LABEL in tags_dic:
                    tags_dic[NO_TAG_LABEL] += [host]
                else:
                    tags_dic[NO_TAG_LABEL] = [host]
            for tag in host.tags:
                if tag in tags_dic:
                    tags_dic[tag] += [host]
                else:
                    tags_dic[tag] = [host]

        if NO_TAG_LABEL in tags_dic:
            tag_hosts = tags_dic.pop(NO_TAG_LABEL)
        else:
            tag_hosts = []

        no_tag_iter = self.contents.append(None, [NO_TAG_LABEL, "Total: "+str(len(tag_hosts)), NO_TAG_ID])
        for host in tag_hosts:
            self.contents.append(no_tag_iter, [host.hostname, host.address, ITEM_ID])

        tags_dic_keys = tags_dic.keys()
        tags_dic_keys.sort()

        for tag in tags_dic_keys:
            tag_len = len(tags_dic[tag])
            tag_iter = self.contents.append(None, [tag, "Total: "+str(tag_len), TAG_ID])

            for host in tags_dic[tag]:
                self.contents.append(tag_iter, [host.hostname, host.address, ITEM_ID])

    def add_single_host(self, host):
        if len(host.tags) == 0:
            self.add_host_at_tag(NO_TAG_LABEL, host)
            return

        for tag in host.tags:
            tag_iter = self.find_tag_iter(tag)
            if tag_iter is None:
                # Tag do not exist
                self.add_new_tag(tag)
            self.add_host_at_tag(tag, host)


    def remove_single_host(self, host):
        if len(host.tags) == 0:
            # No tags
            no_tag_iter = self.find_tag_iter(NO_TAG_LABEL)
            host_iter = self.find_host_iter(no_tag_iter, host)
            self.contents.remove(host_iter)
            self.update_tag_count(no_tag_iter, -1)
            return

        for tag in host.tags:
            tag_iter = self.find_tag_iter(tag)
            host_iter = self.find_host_iter(tag_iter, host)
            self.contents.remove(host_iter)
            tag_count = self.update_tag_count(tag_iter, -1)
            if tag_count == 0:
                self.remove_tag(tag)

    def update_single_host(self, host):
        model = self.get_model()
        # If only at 'No Tags' tag
        if len(host.tags) == 0:
            # Ensure host is not in any other tag
            for tag_model in self.contents:
                host_iter = self.find_host_iter(tag_model.iter, host)
                if host_iter is not None:
                    self.contents.remove(host_iter)
                    tag_count = self.update_tag_count(tag_model.iter, -1)
                    if tag_count == 0:
                        self.contents.remove(tag_model.iter)

            self.add_host_at_tag(NO_TAG_LABEL, host)
            return

        # Goes through host tags to add the host
        for tag in host.tags:
            tag_iter = self.find_tag_iter(tag)
            # check for non existing tags
            if tag_iter is None:
                # Add new tags, host and update the counter
                tag_iter = self.add_new_tag(tag)
                self.add_host_at_tag(tag, host)
                continue
            # Tries to find host at tag
            host_iter = self.find_host_iter(tag_iter, host)
            # Check if host isn't at that tag
            if host_iter is None:
                self.add_host_at_tag(tag, host)

        # Goes throught host tags clean old tags
        for tag_model in self.contents:
            tag_name = model.get(tag_model.iter, 0)[0]
            # Tags not used
            if tag_name not in host.tags:
                # has that host?
                host_iter = self.find_host_iter(tag_model.iter, host)
                if host_iter is not None:
                    # Remove it
                    self.contents.remove(host_iter)
                    self.update_tag_count(tag_model.iter, -1)


class TreeNotebook(gtk.Notebook):
    __gsignals__ = {
                    'open-connection'      : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'item-selected'        : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                   }
    def __init__(self):
        gtk.Notebook.__init__(self)
        self.set_show_tabs(False)
        self.hosts = lib.db.hosts

        self.contents = [
                         SimpleHostsList(),
                         #GroupHostsList(),
                         TagHostsList(),
                        ]

        for host_list in self.contents:
            host_list.create_list()
            self.append_page(host_list)
            host_list.connect('open-connection', self.open_connection)
            host_list.connect('item-selected', self.item_selected)

        self.set_current_page(0)
        self.show_all()

    def change_page(self, widget, index):
        self.set_current_page(index)

        tree = self.contents[index]
        selection = tree.get_selection()
        selection.unselect_all()
        tree.emit('item-selected', None)

    def get_showing_tree(self):
        return self.contents[self.get_current_page()]

    def open_connection(self, widget, host):
        self.emit('open-connection', host)

    def filter(self, string):
        pass

    def item_selected(self, widget, item):
        self.emit('item-selected', item)
