#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject

import beholderlib as lib
from beholderlib.hosts import BIG_ICONS_PATH

_ = lib._
PREVIEW_SIZE = 150

class ImageChooserDialog(gtk.FileChooserDialog):
    def __init__(self):
        gtk.FileChooserDialog.__init__(self)
        self.set_title(_("Choose an image..."))
        self.set_current_folder(BIG_ICONS_PATH)

        self.previewidget = gtk.Image()
        self.use_preview = True

        self.toggle = gtk.CheckButton(_("Preview images"))
        self.toggle.set_active(True)
        self.toggle.connect('toggled', self.__on_toggle_pressed)
        self.toggle.show()

        self.set_extra_widget(self.toggle)
        self.set_preview_widget(self.previewidget)
        self.set_use_preview_label(False)

        filter = gtk.FileFilter()
        filter.set_name(_('Images (png)'))
        filter.add_mime_type('image/png')
        self.add_filter(filter)
        self.set_filter(filter)

        self.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
        self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)

        self.connect("update-preview", self.__on_update_preview)

    def set_preview(self):
        filename = self.get_preview_filename()
        try:
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename, PREVIEW_SIZE, PREVIEW_SIZE)
            self.previewidget.set_from_pixbuf(pixbuf)
            self.have_preview = True
        except:
            self.have_preview = False
        self.set_preview_widget_active(self.have_preview)

    def __on_toggle_pressed(self, toggle):
        if toggle.get_active():
            self.set_preview()
            self.use_preview = True
        else:
            self.set_preview_widget_active(False)
            self.use_preview = False

    def __on_update_preview(self, file_chooser):
        if not self.use_preview:
            return
        self.set_preview()

    def show_dialog(self):
        ret = self.run()
        if ret == gtk.RESPONSE_OK:
            return self.get_preview_filename()
        else:
            return None

