#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject

class Table(gtk.Table):
    def __init__(self, rows=5):
        gtk.Table.__init__(self, rows=rows, columns=3)
        self.set_row_spacings(2)
        self.set_col_spacings(5)
        self.pos = 0
        self.left_padding = 15

    def add_empty_row(self):
        label = gtk.Label("")
        self.attach(label, 0, 3, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.pos += 1

    def add_row_title(self, title, bold=True):
        label = gtk.Label()
        if bold:
            label.set_markup("<b>"+title+"</b>")
        else:
            label.set_markup(title)
        label.set_alignment(0, 0.5)
        self.attach(label, 0, 3, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.pos += 1

    def add_row_widget(self, label_text, widget):
        label = gtk.Label()
        label.set_markup(label_text)
        label.set_alignment(0, 0.5)
        self.attach(label, 0, 1, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=self.left_padding, ypadding=0)
        self.attach(gtk.Label(" : "), 1, 2, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.attach(widget, 2, 3, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.pos += 1
        
    def add_double_row_widget(self, upper_text, bottom_text, widget):
        upper_label = gtk.Label()
        upper_label.set_markup(upper_text + '\n' + bottom_text)
        upper_label.set_alignment(0, 0)
        self.attach(upper_label, 0, 1, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=self.left_padding, ypadding=0)
                    
        #bottom_label = gtk.Label()
        #bottom_label.set_markup(bottom_text)
        #bottom_label.set_alignment(0, 0)
        #self.attach(bottom_label, 0, 1, self.pos+1, self.pos+2,
        #            xoptions=gtk.EXPAND|gtk.FILL,
        #            yoptions=gtk.EXPAND|gtk.FILL,
        #            xpadding=15, ypadding=0)
                    
        self.attach(gtk.Label(" : "), 1, 2, self.pos, self.pos+2,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.attach(widget, 2, 3, self.pos, self.pos+2,
                    xoptions=gtk.EXPAND|gtk.FILL,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.pos += 2

    def add_single_widget_row(self, widget, shrink=True):
        if shrink:
            self.attach(widget, 0, 3, self.pos, self.pos+1,
                        xoptions=gtk.EXPAND,
                        yoptions=gtk.EXPAND|gtk.FILL,
                        xpadding=0, ypadding=0)
        else:
            self.attach(widget, 0, 3, self.pos, self.pos+1,
                        xoptions=gtk.EXPAND|gtk.FILL,
                        yoptions=gtk.EXPAND|gtk.FILL,
                        xpadding=0, ypadding=0)
        self.pos += 1
        
    def add_separator(self):
        sep = gtk.HSeparator()
        self.attach(sep, 0, 3, self.pos, self.pos+1,
                    xoptions=gtk.EXPAND,
                    yoptions=gtk.EXPAND|gtk.FILL,
                    xpadding=0, ypadding=0)
        self.pos += 1


class Notebook(gtk.Notebook):
    def __init__(self):
        gtk.Notebook.__init__(self)
        
    def append_page(self, tab_widget, tab_label):
        label = gtk.Label()
        label.set_markup('<b>'+tab_label+'</b>')
        
        align = gtk.Alignment(xalign=0, yalign=0.0, xscale=0.0, yscale=0.0)
        align.add(tab_widget)
        align.set_padding(padding_top=10, padding_bottom=10, padding_left=10, padding_right=10)
        gtk.Notebook.append_page(self, align, label)
        
