#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject
from os.path import basename
import beholderlib as lib

widgets = lib.widgets.hostproperties

SMALL_SIZE_INT = lib.hosts.SMALL_SIZE_INT
MEDIUM_SIZE_INT = lib.hosts.MEDIUM_SIZE_INT
BIG_SIZE_INT = lib.hosts.BIG_SIZE_INT
SMALL_ICONS_PATH = lib.hosts.SMALL_ICONS_PATH
MEDIUM_ICONS_PATH = lib.hosts.MEDIUM_ICONS_PATH
BIG_ICONS_PATH = lib.hosts.BIG_ICONS_PATH

_ = lib._

class HostPropertiesWindow(gtk.Dialog):
    def __init__(self, master_window, host=None, title=_('New host')):
        gtk.Dialog.__init__(self, title, master_window,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        self.host = host
        self.set_resizable(False)
        #self.set_icon_from_file(WINDOW_ICON)
        self.set_border_width(10)
        self.vbox.set_spacing(10)

        # Contents
        self.icon = widgets.HostIcon()
        align = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0.0)
        align.add(self.icon)
        self.vbox.pack_start(align, fill=False, expand=False)

        self.notebook = lib.widgets.misc.Notebook()
        self.vbox.pack_start(self.notebook)

        ### General
        self.table_general = lib.widgets.misc.Table(rows=3)
        #self.table_general.left_padding = 15
        self.notebook.append_page(self.table_general, _("General"))

        # General
        self.table_general.add_row_title(_('General'))
        self.hostname_entry = widgets.HostnameEntry()
        self.hostname_entry.connect('changed-hostname', self.__on_property_change)
        self.table_general.add_row_widget(_("Hostname"), self.hostname_entry)

        hostname_alert = lib.widgets.misc.HostnameAlert()
        self.hostname_entry.connect('alert-hostname', hostname_alert.on_trigger)
        self.table_general.add_single_widget_row(hostname_alert)

        self.address_entry = widgets.HostAddress()
        self.address_entry.connect('changed-address', self.__on_property_change)
        self.table_general.add_row_widget(_("Address"), self.address_entry)

        # Tag
        self.table_general.add_empty_row()
        self.table_general.add_row_title('<b>'+_('Tags')+'</b> <small>('+_('Comma separated')+')</small>', bold=False)

        self.tag_entry = widgets.HostTags()
        self.tag_entry.set_size_request(-1, 55)
        align = gtk.Alignment(xalign=1, yalign=1, xscale=1, yscale=1)
        align.set_padding(padding_top=0, padding_bottom=0, padding_left=10, padding_right=0)
        align.add(self.tag_entry)
        self.table_general.add_single_widget_row(align, shrink=False)


        ### Connections
        self.table_connections = lib.widgets.misc.Table(rows=3)
        self.notebook.append_page(self.table_connections, _("Connection"))

        # General
        self.table_connections.add_row_title(_('General'))
        self.username_entry = gtk.Entry()
        self.table_connections.add_row_widget(_("Username"), self.username_entry)

        # Connection
        self.table_connections.add_empty_row()
        self.table_connections.add_row_title(_('Connection'))
        self.protocol_combo = widgets.ProtocolComboBox(ports=False)
        self.table_connections.add_row_widget(_("Protocol"), self.protocol_combo)

        self.port_spin = widgets.PortSpin()
        self.protocol_combo.connect('protocol-changed', self.port_spin.update_with_protocol)
        self.table_connections.add_row_widget(_("Port"), self.port_spin)

        ser = self.protocol_combo.get_selected()
        self.port_spin.update_with_protocol(None, ser)

        # Buttons
        self.apply = self.add_button(gtk.STOCK_APPLY, gtk.RESPONSE_ACCEPT)
        self.close = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_REJECT)
        self.apply.set_sensitive(False)

        self.show_all()

        hostname_alert.on_trigger(None, False)

        if self.host is not None:
            self.load_from_host()

    def __on_property_change(self, widget):
        # TODO: Fix it all
        if self.hostname_entry.is_valid and self.address_entry.is_valid:
            self.apply.set_sensitive(True)
        else:
            self.apply.set_sensitive(False)


    def process_image(self):
        icon = self.get_icon_dest_path()
        pixbuf_small = gtk.gdk.pixbuf_new_from_file_at_size(self.icon.image_path, SMALL_SIZE_INT, SMALL_SIZE_INT)
        pixbuf_small.save(SMALL_ICONS_PATH + icon, "png")

        pixbuf_medium = gtk.gdk.pixbuf_new_from_file_at_size(self.icon.image_path, MEDIUM_SIZE_INT, MEDIUM_SIZE_INT)
        pixbuf_medium.save(MEDIUM_ICONS_PATH + icon, "png")

        pixbuf_big = gtk.gdk.pixbuf_new_from_file_at_size(self.icon.image_path, BIG_SIZE_INT, BIG_SIZE_INT)
        pixbuf_big.save(BIG_ICONS_PATH + icon, "png")
        return icon

    def load_from_host(self):
        self.icon.load_from_file(self.host.get_big_icon())
        self.hostname_entry.set_text(self.host.hostname)
        self.hostname_entry.set_sensitive(False)
        self.hostname_entry.is_valid = True

        self.address_entry.set_text(self.host.address)
        self.address_entry.is_valid = True
        self.__on_property_change(None)

        self.tag_entry.load_tags(self.host.tags)

        if self.host.connection.username not in (None, ""):
            self.username_entry.set_text(self.host.connection.username)
        self.protocol_combo.set_protocol(self.host.connection.type)
        self.port_spin.set_port(self.host.connection.port)

    def get_icon_dest_path(self):
        image_path = self.icon.image_path
        icon = basename(self.icon.image_path).split('.')[:-1]+['png']
        icon = '.'.join(icon)
        return icon

    def get_connection(self):
        connection = {
                      'type':self.protocol_combo.get_selected(),
                      'port':self.port_spin.get_port(),
                     }
        if self.username_entry.get_text() != "":
            connection['username'] = self.username_entry.get_text().strip()
        else:
            connection['username'] = None
        return connection


    def run(self):
        if gtk.Dialog.run(self) in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT):
            return None

        if self.host is not None:
            # Changing an existing host properties
            new_host_dic = {}
            if self.host.address != self.address_entry.get_text():
                new_host_dic['address'] = self.address_entry.get_text()
            if self.host.tags != self.tag_entry.get_tags():
                new_host_dic['tags'] = self.tag_entry.get_tags()
            if self.host.connection.as_dict() != self.get_connection():
                new_host_dic['connection'] = self.get_connection()
            if self.icon.image_path is not None \
            and self.host.icon != self.get_icon_dest_path():
                new_host_dic['icon'] = self.process_image()
            return new_host_dic

        new_host_dic = {
                        'hostname':self.hostname_entry.get_text(),
                        'address':self.address_entry.get_text(),
                        'tags':self.tag_entry.get_tags(),

                        'connection':self.get_connection(),
                       }

        if self.icon.image_path is not None:
            new_host_dic['icon'] = self.process_image()

        return new_host_dic
