#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject
import beholderlib as lib

widgets = lib.widgets.mainwindow

ITEM_ID = lib.hosts.ITEM_ID
ROOT_ID = lib.hosts.ROOT_ID
GROUP_ID = lib.hosts.GROUP_ID
NO_GROUP_ID = lib.hosts.NO_GROUP_ID
TAG_ID = lib.hosts.TAG_ID
NO_TAG_ID = lib.hosts.NO_TAG_ID
ICONS_PATH = lib.ICONS_PATH
SEP = lib.hosts.SEP

class MainWindow(gobject.GObject):
    __gsignals__ = {
                    'quickly-add-host'  : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'add-host'          : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
                    'update-host'       : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'remove-host'       : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
                    'host-properties'   : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
                   }
    def __init__(self):
        gobject.GObject.__init__(self)
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.connect("destroy", self.__on_destroy)
        self.window.set_border_width(10)
        self.window.set_size_request(800,500)
        self.window.set_title("Beholder")
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_icon_from_file(ICONS_PATH + SEP + '64x64' + SEP +'beholder.png')

        self.db = lib.db
        self.add_widgets()
        self.window.show_all()

    def add_widgets(self):
        self.main_vbox = gtk.VBox()
        self.window.add(self.main_vbox)

        self.main_hpanel = gtk.HPaned()
        self.main_vbox.pack_start(self.main_hpanel)

        # Left pannel box
        self.left_vbox = gtk.VBox(spacing=5)
        self.main_hpanel.pack1(self.left_vbox)

        self.host_listcombo = widgets.HostListCombo()
        self.left_vbox.pack_start(self.host_listcombo, expand=False)

        self.tree_notebook = widgets.TreeNotebook()
        self.host_listcombo.connect('group-selected', self.tree_notebook.change_page)
        self.left_vbox.pack_start(self.tree_notebook)

        self.small_toolbar = widgets.SmallToolbar()
        self.small_toolbar.connect('quickly-add-host', self.__on_quickly_add_host)
        self.small_toolbar.connect('remove-host', self.__on_remove_host)
        self.small_toolbar.connect('add-host', self.__on_add_host)
        self.small_toolbar.connect('host-properties', self.__on_host_properties)

        self.tree_notebook.connect('item-selected', self.small_toolbar.on_item_selected)
        self.small_toolbar.on_item_selected(None, None)

        self.left_vbox.pack_start(self.small_toolbar, expand=False)

        self.conn_notebook = widgets.ConnNotebook()
        self.tree_notebook.connect('open-connection', self.conn_notebook.open_connection)
        self.main_hpanel.pack2(self.conn_notebook)
        
        self.host_popupmenu = widgets.popupmenus.host_popupmenu
        self.host_popupmenu.connect('remove-host', self.__on_remove_host)
        self.host_popupmenu.connect('host-properties', self.__on_host_properties)
        self.host_popupmenu.connect('add-host', self.__on_add_host)
        self.host_popupmenu.connect('quickly-add-host', self.__on_quickly_add_host)


    def __on_quickly_add_host(self, widget):
        self.emit('quickly-add-host')

    def __on_remove_host(self, widget):
        tree = self.tree_notebook.get_showing_tree()
        model = tree.get_selected()
        if model is None:
            return
        elif model[-1] == ITEM_ID:
            self.emit('remove-host', model)
            

    def __on_add_host(self, widget):
        self.emit('add-host')

    def __on_destroy(self, widget):
        gtk.main_quit()

    def __on_host_properties(self, widget):
        tree = self.tree_notebook.get_showing_tree()
        model = tree.get_selected()
        if model is None:
            return
        elif model[-1] == ITEM_ID:
            self.emit('host-properties', model)

    def run(self):
        gtk.main()

