#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gtk
import pygtk
import gobject
from os.path import basename
import beholderlib as lib

widgets = lib.widgets.hostproperties

SMALL_SIZE_INT = lib.hosts.SMALL_SIZE_INT
MEDIUM_SIZE_INT = lib.hosts.MEDIUM_SIZE_INT
BIG_SIZE_INT = lib.hosts.BIG_SIZE_INT
SMALL_ICONS_PATH = lib.hosts.SMALL_ICONS_PATH
MEDIUM_ICONS_PATH = lib.hosts.MEDIUM_ICONS_PATH
BIG_ICONS_PATH = lib.hosts.BIG_ICONS_PATH

_ = lib._

class QuicklyAddHost(gtk.Dialog):
    def __init__(self, master_window):
        gtk.Dialog.__init__(self, _("Quickly Add Host"), master_window,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        self.set_resizable(False)
        #self.set_icon_from_file(WINDOW_ICON)
        self.set_border_width(10)
        self.vbox.set_spacing(10)

        # Contents
        #self.table = gtk.Table(rows=5, columns=3)
        self.table = lib.widgets.misc.Table(rows=5)
        #self.table.set_row_spacings(2)
        #self.table.set_col_spacings(5)
        self.vbox.pack_start(self.table)

        self.icon = widgets.HostIcon()
        self.table.add_single_widget_row(self.icon)

        self.table.add_empty_row()
        self.table.add_row_title(_("General Options"))

        self.hostname_entry = widgets.HostnameEntry()
        self.hostname_entry.connect('changed-hostname', self.__on_property_change)
        self.table.add_row_widget(_("Hostname"), self.hostname_entry)

        hostname_alert = lib.widgets.misc.HostnameAlert()
        self.hostname_entry.connect('alert-hostname', hostname_alert.on_trigger)
        self.table.add_single_widget_row(hostname_alert)

        self.address_entry = widgets.HostAddress()
        self.address_entry.connect('changed-address', self.__on_property_change)
        self.table.add_row_widget(_("Address"), self.address_entry)
        self.table.add_empty_row()

        self.table.add_row_title(_("Connection Options"))

        self.username_entry = gtk.Entry()
        self.table.add_row_widget(_("Username"), self.username_entry)

        self.protocol_combo = widgets.ProtocolComboBox()
        self.table.add_row_widget(_("Protocol"), self.protocol_combo)

        # Buttons
        self.apply = self.add_button(gtk.STOCK_APPLY, gtk.RESPONSE_ACCEPT)
        self.close = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_REJECT)
        self.apply.set_sensitive(False)

        self.show_all()

        hostname_alert.on_trigger(None, False)

    def __on_property_change(self, widget):
        # TODO: Fix it all
        if self.hostname_entry.is_valid and self.address_entry.is_valid:
            self.apply.set_sensitive(True)
        else:
            self.apply.set_sensitive(False)

    def process_image(self):
        image_path = self.icon.image_path
        icon = basename(self.icon.image_path).split('.')[:-1]+['png']
        icon = '.'.join(icon)
        pixbuf_small = gtk.gdk.pixbuf_new_from_file_at_size(image_path, SMALL_SIZE_INT, SMALL_SIZE_INT)
        pixbuf_small.save(SMALL_ICONS_PATH + icon, "png")

        pixbuf_medium = gtk.gdk.pixbuf_new_from_file_at_size(image_path, MEDIUM_SIZE_INT, MEDIUM_SIZE_INT)
        pixbuf_medium.save(MEDIUM_ICONS_PATH + icon, "png")

        pixbuf_big = gtk.gdk.pixbuf_new_from_file_at_size(image_path, BIG_SIZE_INT, BIG_SIZE_INT)
        pixbuf_big.save(BIG_ICONS_PATH + icon, "png")
        return icon

    def run(self):
        if gtk.Dialog.run(self) in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT):
            return None

        new_host_dic = {
                        'hostname':self.hostname_entry.get_text(),
                        'address':self.address_entry.get_text(),

                       }
        if self.icon.image_path is not None:
            new_host_dic['icon'] = self.process_image()

        (service, port) = self.protocol_combo.get_selected()
        new_host_dic['connection'] = {'type':service, 'port':port}

        if self.username_entry.get_text() != "":
            new_host_dic['connection']['username'] = self.username_entry.get_text()

        return new_host_dic
