#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Magnun Leno da Silva
#
# This file is part of Beholder
#
# Beholder is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Beholder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Beholder.  If not, see <http://www.gnu.org/licenses/>.
#

import gobject

import os
from sys import path as PATH
from os import sep as SEP

import gui
import beholderlib as lib

_ = lib._


class Main(gobject.GObject):
    def __init__(self):
        gobject.GObject.__init__(self)
        self.mainwindow = gui.MainWindow()
        self.mainwindow.connect('remove-host', self.__on_remove_host)
        self.mainwindow.connect('quickly-add-host', self.__on_quickly_add_host)
        self.mainwindow.connect('add-host', self.__on_add_host)
        self.mainwindow.connect('host-properties', self.__on_host_properties)

    def get_version(self):
        ver_f_name = SEP.join(__file__.split(SEP)[:-1]+['version'])
        return open(ver_f_name).readline().strip()

    def __on_remove_host(self, widget, host):
        lib.db.hosts.remove(host[0])

    def __on_quickly_add_host(self, widget):
        quickly_add = gui.QuicklyAddHost(self.mainwindow.window)
        new_host = quickly_add.run()
        quickly_add.destroy()

        if new_host is None:
            return
        lib.db.hosts.update(**new_host)

    def __on_add_host(self, widget):
        host_add = gui.HostPropertiesWindow(self.mainwindow.window)
        new_host = host_add.run()
        host_add.destroy()
        if new_host is None:
            return

        lib.db.hosts.update(**new_host)

    def __on_host_properties(self, widget, host_data):
        host = lib.db.hosts.get_by_hostname(host_data[0])
        host_add = gui.HostPropertiesWindow(self.mainwindow.window, host=host, title="'"+host_data[0]+"' "+_("Properties"))
        host_prop = host_add.run()
        host_add.destroy()

        if host_prop == {} or host_prop is None:
            return
        
        host.update_by_dict(host_prop)

    def run(self):
        self.mainwindow.run()

def run():
    main = Main()
    main.run()
